/**
 * Created by Sam R.
 * on 17-12-2011
 * at 12:43
 */
package com.cirrial.ld22 {
import com.cirrial.ld22.widgets.Powerable;

import org.flixel.FlxG;
import org.flixel.FlxGroup;
import org.flixel.FlxSprite;

public class Player extends FlxSprite {
    [Embed(source = '../../../../data/graphics/player.png')]
    private var ImgPlayer:Class;

    // constants
    public static const INIT_ENERGY:Number = 50;
    public static const MAX_ENERGY:Number = 100;
    public static const MOVE_ACCEL:Number = 250;
    public static const JUMP_STRENGTH_MAX:Number = 100;
    public static const JUMP_DECAY:Number = 30;
    public static const MAX_POWER_RANGE:Number = 60;

    // variables
    private var jumping:Boolean = false;
    private var jumpStrength:Number = 0;

    public var energy:Number;
    public var isDying:Boolean = false;
    public var isPowering:Boolean = false;
    public var isHurt:Boolean = false;

    public var isTransition:Boolean = false;
    public var isTransitioned:Boolean = false;

    public var powerables:FlxGroup;

    public function Player(X:Number, Y:Number) {
        super(X, Y);
        loadGraphic(ImgPlayer, true, true, 40, 45);

        width = 19;
        height = 39;
        offset.x = 14;
        offset.y = 6;

        acceleration.y = Main.GRAVITY;
        maxVelocity.x = 150;
        maxVelocity.y = 400;
        drag.x = 1000;
        energy = INIT_ENERGY;

        addAnimation("stand", [0, 1, 2, 3, 4, 5], 6, true);
        addAnimation("walk", [6, 7, 8, 9, 10, 11], 8, true);
        addAnimation("powerdowntrans", [12, 13, 14], 6, false);
        addAnimation("powerdown", [15, 16, 17, 16], 6, true);
        addAnimation("poweruptrans", [18, 19, 20], 6, false);
        addAnimation("powerup", [21, 22, 23, 22], 6, true);
        addAnimation("die", [24, 25, 26, 27, 28, 29], 6, false);
        addAnimation("hurt", [30, 31, 32], 6, false);
        addAnimation("jump", [33, 34, 35], 6, false);
        addAnimation("fall", [36, 37, 38], 6, true);
    }

    public function getPowerables(powerables:FlxGroup):void {
        this.powerables = powerables;
    }

    override public function update():void {

        if(isDying) {
            velocity.x = 0;

            return;
        }

        if(isHurt) {
            if(finished) {
                isHurt = false;
            }
            return;
        }

        // move on x axis
        if(!isPowering) {
            if(FlxG.keys.pressed(Main.KEY_RIGHT)) {
                acceleration.x = MOVE_ACCEL;
                if(velocity.x < 0) velocity.x = 0;
                facing = RIGHT;
                offset.x = 14;
            } else if(FlxG.keys.pressed(Main.KEY_LEFT)) {
                acceleration.x = -MOVE_ACCEL;
                if(velocity.x > 0) velocity.x = 0;
                facing = LEFT;
                offset.x = 6;
            } else {
                acceleration.x = 0;
            }

            // jump
            if((FlxG.keys.justPressed(Main.KEY_JUMP))) {
                if(!jumping && isTouching(DOWN)) {
                    jumping = true;
                    jumpStrength = JUMP_STRENGTH_MAX;
                }
            }
            if(jumping) {
                if((FlxG.keys.pressed(Main.KEY_JUMP))) {
                    if(jumpStrength > 0) {
                        velocity.y -= jumpStrength * (Main.EXPECTED_TIMESTEP * 100);
                        jumpStrength -= JUMP_DECAY * (Main.EXPECTED_TIMESTEP * 100);
                    }
                } else {
                    jumping = false;
                }
            }
            if(velocity.y > 0) {
                play("fall");
            } else if(velocity.y < 0) {
                play("jump");
            } else if(velocity.x != 0) {
                play("walk");
            } else {
                play("stand");
            }
        }

        // power manipulation

        if(FlxG.keys.pressed(Main.KEY_DOWN) || FlxG.keys.pressed(Main.KEY_UP)) {
            // intending to do a thing to a nearby powerable object
            var target:Powerable = Main.getClosestObject(this, powerables, MAX_POWER_RANGE) as Powerable;
            if(target) {
                isPowering = true;
                velocity.x = 0;
                acceleration.x = 0;
                var adjustment:Number = Main.TRANSFER_RATE * Main.EXPECTED_TIMESTEP;

                if(FlxG.keys.pressed(Main.KEY_DOWN)) {
                    // put energy into widget
                    if(energy >= adjustment && !target.powered) {
                        adjustEnergy(-adjustment);
                        target.adjustEnergy(adjustment);
                    }

                    // animation control
                    if(!isTransitioned) {
                        if(!isTransition) {
                            isTransition = true;
                            play("powerdowntrans");
                            Main.playSound(SoundData.SndPowerDown, this);
                        } else if(finished) {
                            isTransitioned = true;
                            play("powerdown");
                        }
                    }


                } else if(FlxG.keys.pressed(Main.KEY_UP)) {
                    // get energy out of widget
                    if(target.energy < adjustment) {
                        adjustment = target.energy;
                    }
                    adjustEnergy(adjustment);
                    target.adjustEnergy(-adjustment);

                    // animation control
                    if(!isTransitioned) {
                        if(!isTransition) {
                            isTransition = true;
                            play("poweruptrans");
                            Main.playSound(SoundData.SndPowerUp, this);
                        } else if(finished) {
                            isTransitioned = true;
                            play("powerup");
                        }
                    }

                }
            }
        } else {
            isPowering = false;
            isTransition = false;
            isTransitioned = false;
        }
    }

    public function adjustEnergy(energyAmount:Number):void {
        energy += energyAmount;
        if(energy > MAX_ENERGY) energy = MAX_ENERGY;
        else if(energy < 0) {
            alive = false;
            isDying = true;
            play("die");
            Main.playSound(SoundData.SndDie, this);
        }
    }

    override public function hurt(amount:Number):void {
        adjustEnergy(-amount);
        isHurt = true;
        play("hurt");
    }
}
}
